from clikit.config import DefaultApplicationConfig

from cleo.io import ConsoleIO


class ApplicationConfig(DefaultApplicationConfig):
    """
    Cleo's default application configuration.
    """

    def configure(self):  # type: () -> None
        super(ApplicationConfig, self).configure()

    @property
    def io_class(self):
        return ConsoleIO



from typing import Any
from typing import Optional

from clikit.api.args.format import Argument
from clikit.api.args.format import Option


def argument(
    name, description=None, optional=False, multiple=False, default=None
):  # type: (str, Optional[str], bool, bool, Optional[Any]) -> Argument
    if optional:
        flags = Argument.OPTIONAL
    else:
        flags = Argument.REQUIRED

    if multiple:
        flags |= Argument.MULTI_VALUED

    return Argument(name, flags, description, default)


def option(
    long_name,
    short_name=None,
    description=None,
    flag=True,
    value_required=True,
    multiple=False,
    default=None,
):  # type: (str, Optional[str], Optional[str], bool, bool, bool, Optional[Any]) -> Option
    if flag:
        flags = Option.NO_VALUE
    elif value_required:
        flags = Option.REQUIRED_VALUE
    else:
        flags = Option.OPTIONAL_VALUE

    if multiple and not flag:
        flags |= Option.MULTI_VALUED

    return Option(long_name, short_name, flags, description, default)



from clikit.args import StringArgs

from cleo import Application
from cleo.io import BufferedIO


class ApplicationTester(object):
    """
    Eases the testing of console applications.
    """

    def __init__(self, application):  # type: (Application) -> None
        self._application = application
        self._application.config.set_terminate_after_run(False)
        self._io = BufferedIO()
        self._status_code = 0

    @property
    def io(self):  # type: () -> BufferedIO
        return self._io

    @property
    def status_code(self):  # type: () -> int
        return self._status_code

    def execute(self, args, **options):  # type: (str, ...) -> int
        """
        Executes the command

        Available options:
            * interactive: Sets the input interactive flag
            * verbosity: Sets the output verbosity flag
        """
        args = StringArgs(args)

        if "inputs" in options:
            self._io.set_input(options["inputs"])

        if "interactive" in options:
            self._io.set_interactive(options["interactive"])

        if "verbosity" in options:
            self._io.set_verbosity(options["verbosity"])

        self._status_code = self._application.run(
            args,
            self._io.input.stream,
            self._io.output.stream,
            self._io.error_output.stream,
        )

        return self._status_code



# -*- coding: utf-8 -*-

import sys

PY2 = sys.version_info[0] == 2

if PY2:
    long = long
    unicode = unicode
    basestring = basestring
else:
    long = int
    unicode = str
    basestring = str


def decode(string, encodings=None):
    if not PY2 and not isinstance(string, bytes):
        return string

    if PY2 and isinstance(string, unicode):
        return string

    if encodings is None:
        encodings = ["utf-8", "latin1", "ascii"]

    for encoding in encodings:
        try:
            return string.decode(encoding)
        except (UnicodeEncodeError, UnicodeDecodeError):
            pass

    return string.decode(encodings[0], errors="ignore")


def encode(string, encodings=None):
    if not PY2 and isinstance(string, bytes):
        return string

    if PY2 and isinstance(string, str):
        return string

    if encodings is None:
        encodings = ["utf-8", "latin1", "ascii"]

    for encoding in encodings:
        try:
            return string.encode(encoding)
        except (UnicodeEncodeError, UnicodeDecodeError):
            pass

    return string.encode(encodings[0], errors="ignore")



from typing import Optional

from clikit.api.args import Args
from clikit.api.command import Command as CliKitCommand
from clikit.api.config.command_config import CommandConfig

from cleo.io import ConsoleIO


class CommandError(Exception):
    pass


class BaseCommand(object):

    name = None

    description = None

    help = None

    arguments = []
    options = []

    aliases = []

    enabled = True
    hidden = False

    commands = []

    def __init__(self):
        self._application = None

        self._config = CommandConfig(self.name)
        self._config.set_description(self.description)
        self._config.set_help(self.help)
        for argument in self.arguments:
            self._config._format_builder.add_argument(argument)

        for option in self.options:
            self._config._format_builder.add_option(option)

        for alias in self.aliases:
            self._config.add_alias(alias)

        if not self.enabled:
            self._config.disable()

        if self.hidden:
            self._config.hide()

        if self.commands:
            for command in self.commands:
                self.add_sub_command(command)

        self._config.set_handler(self)

    @property
    def config(self):  # type: () -> CommandConfig
        return self._config

    @property
    def application(self):
        return self._application

    def handle(
        self, args, io, command
    ):  # type: (Args, ConsoleIO, CliKitCommand) -> Optional[int]
        raise NotImplementedError()

    def set_application(self, application):
        self._application = application

        for command in self.commands:
            command.set_application(application)

    def add_sub_command(self, command):  # type: (BaseCommand) -> None
        self._config.add_sub_command_config(command.config)

        command.set_application(self.application)

    def default(self, default=True):  # type: (bool) -> BaseCommand
        self._config.default(default)

        return self

    def anonymous(self):  # type: () -> BaseCommand
        self._config.anonymous()

        return self



print("import: 'cleo'")
import cleo

print("import: 'cleo.commands'")
import cleo.commands

print("import: 'cleo.commands.completions'")
import cleo.commands.completions

print("import: 'cleo.config'")
import cleo.config

print("import: 'cleo.io'")
import cleo.io

print("import: 'cleo.testers'")
import cleo.testers




# -*- coding: utf-8 -*-

from .application import Application
from .commands import Command
from .helpers import argument, option
from .testers import ApplicationTester, CommandTester


__version__ = "0.8.1"



from clikit.api.command import Command
from clikit.args import StringArgs
from clikit.formatter import AnsiFormatter

from cleo.commands import BaseCommand
from cleo.io import BufferedIO


class CommandTester(object):
    """
    Eases the testing of console commands.
    """

    def __init__(self, command):  # type: (BaseCommand) -> None
        """
        Constructor
        """
        self._command = command
        self._io = BufferedIO()
        self._inputs = []
        self._status_code = None

        if self._command.application:
            for style in self._command.application.config.style_set.styles.values():
                self._io.output.formatter.add_style(style)
                self._io.error_output.formatter.add_style(style)

    @property
    def io(self):  # type: () -> BufferedIO
        return self._io

    @property
    def status_code(self):  # type: () -> int
        return self._status_code

    def execute(self, args="", **options):  # type: (str, ...) -> int
        """
        Executes the command

        Available options:
            * interactive: Sets the input interactive flag
            * decorated: Sets the output decorated flag
            * verbosity: Sets the output verbosity flag
        """
        args = StringArgs(args)

        if "inputs" in options:
            self._io.set_input(options["inputs"])

        if "interactive" in options:
            self._io.set_interactive(options["interactive"])

        if "verbosity" in options:
            self._io.set_verbosity(options["verbosity"])

        if "decorated" in options and options["decorated"]:
            self._io.set_formatter(AnsiFormatter(forced=True))

        command = Command(self._command.config, self._command.application)

        self._status_code = command.run(args, self._io)

        return self._status_code



from typing import Optional
from typing import Tuple

from clikit.console_application import ConsoleApplication

from .commands import BaseCommand
from .commands.completions_command import CompletionsCommand
from .config import ApplicationConfig


class Application(ConsoleApplication, object):
    """
    An Application is the container for a collection of commands.

    This class is optimized for a standard CLI environment.

    Usage:
    >>> app = Application('myapp', '1.0 (stable)')
    >>> app.add(HelpCommand())
    >>> app.run()
    """

    def __init__(
        self, name=None, version=None, complete=True, config=None
    ):  # type: (str, str, bool, Optional[ApplicationConfig]) -> None
        if config is None:
            config = ApplicationConfig(name, version)

        super(Application, self).__init__(config)

        if complete:
            self.add(CompletionsCommand())

    def add_commands(self, *commands):  # type: (Tuple[BaseCommand]) -> None
        for command in commands:
            self.add(command)

    def add(self, command):  # type: (BaseCommand) -> Application
        """
        Adds a command object.
        """
        self.add_command(command.config)
        command.set_application(self)

        return self

    def find(self, name):  # type: (str) -> BaseCommand
        names = name.split(" ")
        command = self.get_command(names[0])
        for name in names[1:]:
            command = command.get_sub_command(name)

        return command.config.handler



from clikit.io import ConsoleIO as BaseConsoleIO

from .io_mixin import IOMixin


class ConsoleIO(IOMixin, BaseConsoleIO):
    """
    A wrapper around CliKit's ConsoleIO.
    """

    def __init__(self, *args, **kwargs):
        super(ConsoleIO, self).__init__(*args, **kwargs)
